from cloudy_warehouses.snowflake_objects.snowflake_object import SnowflakeObject


# Copier Object
class SnowflakeCopier(SnowflakeObject):
    """Class that holds the clone and clone_empty methods."""

    # sql that is run by the cursor object
    sql_statement = str

    def clone(self, database: str, schema: str, new_table: str, source_table: str, source_database: str = None,
              source_schema: str = None, sf_username: str = None, sf_password: str = None, sf_account: str = None):
        """method that creates a copy of a Snowflake table."""
        try:
            # initialize Snowflake connection and configure credentials
            self.initialize_snowflake(
                database=database,
                schema=schema,
                sf_username=sf_username,
                sf_password=sf_password,
                sf_account=sf_account
            )

            # build sql statement to be executed by the cursor object
            if source_database and source_schema:
                self.sql_statement = f"CREATE OR REPLACE TABLE {database}.{schema}.{new_table} CLONE " \
                                     f"{source_database}.{source_schema}.{source_table}"

            elif source_schema and not source_database:
                self.sql_statement = f"CREATE OR REPLACE TABLE {database}.{schema}.{new_table} CLONE " \
                                     f"{source_schema}.{source_table}"

            elif not source_schema and not source_database:
                self.sql_statement = f"CREATE OR REPLACE TABLE {database}.{schema}.{new_table} CLONE " \
                                     f"{source_table}"

            else:
                self.log_message = "Error: please call this method with viable values. Example: If you call this " \
                                   "method with the 'source_database' parameter, " \
                                   "you must include a 'source_schema' parameter as well"
                self._logger.error(self.log_message)
                return False

            # execute sql statement
            self.cursor = self.connection.cursor()
            self.cursor.execute(self.sql_statement)

        # catch and log error
        except Exception as e:
            self.log_message = e
            self._logger.error(self.log_message)
            return False

        finally:
            # close connection and cursor
            if self.connection:
                self.connection.close()
            if self.cursor:
                self.cursor.close()

        # log successful clone
        self.log_message = f"Successfully cloned {source_table} into {database}.{schema}.{new_table}"
        self._logger.info(self.log_message)
        return True

    def clone_empty(self, database: str, schema: str, new_table: str, source_table: str, source_database: str = None,
                    source_schema: str = None, sf_username: str = None, sf_password: str = None,
                    sf_account: str = None):
        """method that creates an empty copy of a Snowflake table."""

        try:
            # initialize Snowflake connection and configure credentials
            self.initialize_snowflake(
                database=database,
                schema=schema,
                sf_username=sf_username,
                sf_password=sf_password,
                sf_account=sf_account
            )

            # build sql statement to be executed by the cursor object
            if source_database and source_schema:
                self.sql_statement = f"CREATE OR REPLACE TABLE {database}.{schema}.{new_table} LIKE " \
                                     f"{source_database}.{source_schema}.{source_table}"

            elif source_schema and not source_database:
                self.sql_statement = f"CREATE OR REPLACE TABLE {database}.{schema}.{new_table} LIKE " \
                                     f"{source_schema}.{source_table}"

            elif not source_schema and not source_database:
                self.sql_statement = f"CREATE OR REPLACE TABLE {database}.{schema}.{new_table} LIKE " \
                                     f"{source_table}"

            else:
                self.log_message = "Error: please call this method with viable values. Example: If you call this " \
                                   "method with the 'source_database' parameter, " \
                                   "you must include a 'source_schema' parameter as well"
                self._logger.error(self.log_message)
                return False

            # execute sql statement
            self.cursor = self.connection.cursor()
            self.cursor.execute(self.sql_statement)

        # catch and log error
        except Exception as e:
            self.log_message = e
            self._logger.error(self.log_message)
            return False

        finally:
            # close connection and cursor
            if self.connection:
                self.connection.close()
            if self.cursor:
                self.cursor.close()

        # log successful clone
        self.log_message = f"Successfully cloned an empty version of {source_table} into " \
                           f"{database}.{schema}.{new_table}"
        self._logger.info(self.log_message)
        return True



import unittest
import mock
import pandas as pd
import cloudy_warehouses


class TestCopySnowflake(unittest.TestCase):
    """Unittests for SnowflakeCopier."""

    def test_clone_snowflake(self):

        # test for success
        mock_clone_snowflake = mock.create_autospec(pd.clone_snowflake, return_value=True)

        self.assertEqual(mock_clone_snowflake(database='a database', schema='a schema',
                                              new_table='a table', source_table='source table',
                                              source_schema='source schema'), True)

        mock_clone_snowflake.assert_called_with(database='a database', schema='a schema',
                                                new_table='a table', source_table='source table',
                                                source_schema='source schema')

        # test for failure
        mock_clone_snowflake = mock.create_autospec(pd.clone_snowflake, return_value=False)

        self.assertEqual(mock_clone_snowflake(database='a database', schema='a schema',
                                              new_table='a table', source_table='source table',
                                              source_database='source database'), False)

        mock_clone_snowflake.assert_called_with(database='a database', schema='a schema',
                                                new_table='a table', source_table='source table',
                                                source_database='source database')

    def test_clone_empty_snowflake(self):

        # test for success
        mock_clone_snowflake = mock.create_autospec(pd.clone_empty_snowflake, return_value=True)

        self.assertEqual(mock_clone_snowflake(database='a database', schema='a schema',
                                              new_table='a table', source_table='source table'), True)

        mock_clone_snowflake.assert_called_with(database='a database', schema='a schema',
                                                new_table='a table', source_table='source table')

        # test for failure
        mock_clone_snowflake = mock.create_autospec(pd.clone_empty_snowflake, return_value=False)

        self.assertEqual(mock_clone_snowflake(database='a database', schema='a schema',
                                              new_table='a table', source_table='source table',
                                              source_database='source database'), False)

        mock_clone_snowflake.assert_called_with(database='a database', schema='a schema',
                                                new_table='a table', source_table='source table',
                                                source_database='source database')




import unittest
import mock
import pandas as pd
import cloudy_warehouses


class TestReadSnowflake(unittest.TestCase):
    """Unittests for SnowflakeReader."""

    def test_read_snowflake(self):

        mock_read_snowflake = mock.create_autospec(pd.read_snowflake,
                                                   return_value=pd.DataFrame.from_dict({
                                                       'COL_1': ['hello', 'there'],
                                                       'COL_2': [10, 1000]
                                                   })
                                                   )

        self.assertIsInstance(mock_read_snowflake(database='a database', schema='a schema', table='a table'),
                              pd.DataFrame)

        mock_read_snowflake.assert_called_with(database='a database', schema='a schema', table='a table')

    def test_list_snowflake(self):

        mock_list_snowflake = mock.create_autospec(pd.list_snowflake_tables,
                                                   return_value=pd.DataFrame.from_dict({
                                                       'COL_1': ['hello', 'there'],
                                                       'COL_2': [10, 1000]
                                                   })
                                                   )

        self.assertIsInstance(mock_list_snowflake(database='a database'), pd.DataFrame)

        mock_list_snowflake.assert_called_with(database='a database')



import yaml
import os
import logging
from snowflake import connector


class SnowflakeObject:
    """Base class for all Snowflake Object types.
    This object holds the methods needed to connect to Snowflake and configure Snowflake credentials."""

    # True once Snowflake configured and connection established
    initialized = False
    # credentials used to establish Snowflake connection
    sf_credentials = None
    # Snowflake connection object
    connection = False
    # cursor object
    cursor = False
    # logger object
    _logger = logging.getLogger()
    # log message
    log_message = str

    def initialize_snowflake(self, database: str = None, schema: str = None, sf_username: str = None, sf_password: str = None,
                             sf_account: str = None):
        """method that creates Snowflake connection and configures Snowflake credentials"""

        if not self.initialized:
            # calls method to configure Snowflake credentials
            self.configure_credentials(
                sf_username=sf_username,
                sf_password=sf_password,
                sf_account=sf_account
            )
            # calls method to connect to Snowflake using the sf_credentials variable
            self.get_snowflake_connection(
                user=self.sf_credentials['user'],
                pswd=self.sf_credentials['pass'],
                acct=self.sf_credentials['acct'],
                database=database,
                schema=schema
            )

        self.initialized = True

    def configure_credentials(self, sf_username: str = None, sf_password: str = None, sf_account: str = None):
        """configures Snowflake credentials"""

        # determines where to look for Snowflake credentials
        if not sf_username or not sf_password or not sf_account:
            # Path to Snowflake credentials file
            __profile_path: str = os.path.join(os.getenv("CLOUDY_HOME"),
                                               '.cloudy_warehouses/configuration_profiles.yml')
            with open(__profile_path, 'r') as stream:
                self.sf_credentials = yaml.safe_load(stream)['profiles']['snowflake']

            # checks if user has configured credentials file
            if self.sf_credentials['user'] == '<your snowflake username>' or self.sf_credentials['pass'] == \
                    '<your snowflake password>' or self.sf_credentials['acct'] == '<your snowflake account>':

                self.log_message = f"Please configure your credentials at {__profile_path} or pass your credentials " \
                                   f"as arguments when calling this method."
                self._logger.error(self.log_message)
                self.sf_credentials = None
                return False

        else:
            # uses passed in variables as Snowflake credentials
            self.sf_credentials = {
                'user': sf_username,
                'pass': sf_password,
                'acct': sf_account
            }

        return True

    def get_snowflake_connection(self, user, pswd, acct, database=None, schema=None):
        """establishes a connection with snowflake"""
        self.connection = connector.connect(
            user=user,
            password=pswd,
            account=acct,
            database=database,
            schema=schema
        )

        return True
